/******************************************************************************/
/*  This file is part of the uVision/ARM development tools                    */
/*  Copyright KEIL ELEKTRONIK GmbH 2002-2004                                  */
/******************************************************************************/
/*                                                                            */
/*  BLINKY.C:  LED Flasher                                                    */
/*                                                                            */
/******************************************************************************/
                  
#include <stdio.h>                         /* standard I/O .h-file */
#include <LPC213x.H>                       /* LPC213x definitions  */
#include "types.h"
#include "mmc.h"
#include "fat16.h"
#include "spi.h"
#include "delay.h"
extern void init_serial (void);            /* Initialize Serial Interface     */

//extern void init_serial (void);            /* Initialize Serial Interface     */

#define STOPBUTTON (1<<14) //P0.14 active low button
#define MAX_BUF 8192

//#define EXT_ADC
#ifdef EXT_ADC
  #define CS_ADC_CHANNEL_0 (1<<6)
  #define CS_ADC_CHANNEL_1 (1<<7)
#endif

u32 volatile smplcnt=0;
s16 adc_buf[MAX_BUF];
#ifdef EXT_ADC
  u32 sstate = 0;
#endif
u32 wrpos=0, rdpos=0;
u32 adcflags=0;
s16 stuff_buf;
u08 sharedmem[512];

extern struct fat_filedata_st fat_filedata;



static void lederror(void) { 
  while (1) 
  { // Error
  	IOSET1 = 0x00FF0000;
	delay_ms(125);
  	IOCLR1 = 0x00FF0000;
	delay_ms(125);
  }
}




// Ints
// Timer Counter 0 Interrupt executes each 10ms @ 60 MHz CPU Clock
void tc0 (void) __irq {
#ifdef EXT_ADC
  sstate = 1;
  IOCLR0 = CS_ADC_CHANNEL_0;
  S0SPDR = 0xff;
#else
  ADGSR = 0x01000000;                          /* Start A/D Conversion */
#endif
  ++smplcnt;

//  DACR = ((Tone.y1 >> 10) & 0xFFC0) + 0x8000;   
  T0IR        = 1;                              /* Clear interrupt flag        */
  VICVectAddr = 0;                              /* Acknowledge Interrupt       */
}

#ifdef EXT_ADC
void spi0 (void) __irq {
  s16 val;
  volatile u32up flush;
  if (sstate == 1 || sstate == 3) {
    S0SPCR = 0x00A4; // MSB first
    S0SPDR = 0;
  }
  else if (sstate == 2 || sstate == 4) {
    IOSET0 = CS_ADC_CHANNEL_0|CS_ADC_CHANNEL_1;
    S0SPCR = 0x08A4; // MSB first
	flush = S0SPSR;
    val = S0SPDR - 32768;
	if (sstate < 4) {
	  IOCLR0 = CS_ADC_CHANNEL_1;
      S0SPDR = 0;
	}
  	adc_buf[wrpos++] = val;
  }
  sstate++;
  S0SPINT = 1;
  VICVectAddr = 0;                              /* Acknowledge Interrupt       */
}
#else
// ADC 0 linker Channel
void adc0 (void) __irq {
  s16 val = (AD0DR & 0xFFC0) - 32768;
  AD0CR &= ~0x01000000;                         /* Stop A/D Conversion */
  adcflags |= 1;
  if (adcflags == 3) {
    ADGSR &= ~0x01000000;                       /* Stop A/D Conversion */
  	adc_buf[wrpos++] = val;
  	adc_buf[wrpos++] = stuff_buf;
	wrpos &= MAX_BUF-1;
#ifdef HALT_ON_ERROR
	if (wrpos==rdpos)
		lederror();
#endif
	adcflags = 0;
  }
  else
    stuff_buf = val;
  VICVectAddr = 0;                              /* Acknowledge Interrupt       */
}

// ADC 1 rechter Channel
void adc1 (void) __irq {
  s16 val = (AD1DR & 0xFFC0) - 32768;
  AD1CR &= ~0x01000000;                         /* Stop A/D Conversion */
  adcflags |= 2;
  if (adcflags == 3) {
    ADGSR &= ~0x01000000;                       /* Stop A/D Conversion */
  	adc_buf[wrpos++] = stuff_buf;
  	adc_buf[wrpos++] = val;
	wrpos &= MAX_BUF-1;
#ifdef HALT_ON_ERROR
	if (wrpos==rdpos)
		lederror();
#endif
	adcflags = 0;
  }
  else
    stuff_buf = val;
  VICVectAddr = 0;                              /* Acknowledge Interrupt       */
}
#endif



/* Setup the Timer Counter 0 Interrupt */
static void init_timer (void) {
  // Timer 0
  T0MR0 = 1874;//1249;                                 /* 48kHz = 1250-1 counts    */
  T0MCR = 3;                                    /* Interrupt and Reset on MR0   */
  T0TCR = 1;                                    /* Timer0 Enable                */
  // Int
  VICVectAddr0 = (unsigned long)tc0;            /* set interrupt vector in 0    */
  VICVectCntl0 = 0x20 | 4;                      /* use it for Timer 0 Interrupt */
  VICIntEnable |= 0x00000010;                   /* Enable Timer0 Interrupt      */
}


/* Setup the ADC */
static void init_adc (void) {
#ifdef EXT_ADC
  PINSEL0 |= 0x00000500; // Pins MISO und SCK von SPI0
  S0SPCR = 0x08A4; // MSB first
  S0SPCCR = 13;
  VICVectAddr1 = (unsigned long)spi0;
  VICVectCntl1 = 0x20 | 10;
  VICIntEnable |= 0x00000400;
#else
  // ADC
  AD0CR = 0x00200D02;                        /* Setup A/D: 10-bit AIN0 @ 4.3MHz */
  AD1CR = 0x00200D01;                        /* Setup A/D: 10-bit AIN1 @ 4.3MHz */
  // Int
  VICVectAddr1 = (unsigned long)adc0;
  VICVectCntl1 = 0x20 | 18;
  VICVectAddr2 = (unsigned long)adc1;
  VICVectCntl2 = 0x20 | 21;
  VICIntEnable |= 0x00640000;
#endif
}


static void write_buffer_word(u08 *buf, u16up b) {
	*buf++ = (u08)b;
	*buf = (u08)(b>>8);
}

static void write_buffer_dword(u08 *buf, u32up b) {
	*buf++ = (u08)b;
	*buf++ = (u08)(b>>8);
	*buf++ = (u08)(b>>16);
	*buf = (u08)(b>>24);
}

static void write_wav_header(void) {
	u32up trklen = smplcnt*4;
	u32up len = trklen + 44 - 8;
	fat_filedata.len = trklen + 44;
	mmc_read_sector(fat_filedata.startsect, sharedmem);

	sharedmem[0]='R';sharedmem[1]='I';sharedmem[2]='F';sharedmem[3]='F';
	write_buffer_dword(&sharedmem[4], len);
	sharedmem[8]='W';sharedmem[9]='A';sharedmem[10]='V';sharedmem[11]='E';

	sharedmem[12]='f';sharedmem[13]='m';sharedmem[14]='t';sharedmem[15]=' ';
	write_buffer_dword(&sharedmem[16], 16);
	write_buffer_word(&sharedmem[20], 1);
	write_buffer_word(&sharedmem[22], 2);
	write_buffer_dword(&sharedmem[24], 48000UL);
	write_buffer_dword(&sharedmem[28], 192000UL);
	write_buffer_word(&sharedmem[32], 4);
	write_buffer_word(&sharedmem[34], 16);

	sharedmem[36]='d';sharedmem[37]='a';sharedmem[38]='t';sharedmem[39]='a';
	write_buffer_dword(&sharedmem[40], trklen);
	mmc_write_sector(fat_filedata.startsect, sharedmem);
}

//extern void simul_spi(void);
int main (void) {
  // Startup: PCLK = CPUCLK = 60MHz
  PCONP = 0x00101512; // PowerDown unbenutzte Peripherie
  IODIR1  = 0x00FF0000;                        /* P1.16..23 defined as LED-Outputs  */
  PINSEL0 = 0x00003000;                        /* enable ADC1 pin */
  PINSEL1 = 0x010000A8;                        /* enable ADC0 pin, SSP */
#ifdef EXT_ADC
  PCONP = 0x00000512; // PowerDown unbenutzte Peripherie
  IODIR0 |=	CS_ADC_CHANNEL_0|CS_ADC_CHANNEL_1;
  IOSET1 = CS_ADC_CHANNEL_0|CS_ADC_CHANNEL_1;
  PINSEL0 = 0x00000000;
  PINSEL1 = 0x000000A8;
#endif

  IOSET1 = 0x00010000;
  init_serial();                               /* Initialize Serial Interface   */
  printf ("\nStartup.\n");       /* Output A/D Conversion Result */
  spi_init();
  MMC_hw_init();
  //simul_spi();
  if (MMC_init()) {
  	IOSET1 = 0x00020000;
	if (fat_init()) {
	  IOSET1 = 0x00040000;
      printf ("\nScanning files.\n");       /* Output A/D Conversion Result */
	  fat_count_files();
	}
	else lederror();
  }
  else lederror();

  fat_openfile(FILEMODE_WRITE, 0);
  mmc_write_start(fat_filedata.startsect);
  mmc_write_block(sharedmem, 44);
  init_adc();
  init_timer();
  IOSET1 = 0x00080000;
  while (IO0PIN & STOPBUTTON) {
    while (wrpos != rdpos) {
	  mmc_write_word(adc_buf[rdpos]);
	  rdpos++; rdpos &= MAX_BUF-1;
	}
  }
  VICIntEnable = 0;
  T0TCR = 0;
  IOSET1 = 0x00100000;
  mmc_complete_write();
  write_wav_header();
  fat_closefile();
  IOSET1 = 0x00200000;
  while (1);                                  /* Loop forever */
}



